@@ -4,13 +4,13 @@ import Pyro4
 print("Showing the different instancing modes.")
 print("The number printed, is the id of the instance that handled the call.")
 
-print("\n-----PERCALL-----")
+print("\n-----PERCALL (different number every time) -----")
 with Pyro4.Proxy("PYRONAME:instance.percall") as p:
     print(p.msg("hello1"))
     print(p.msg("hello2"))
     print(p.msg("hello3"))
 
-print("\n-----SESSION-----")
+print("\n-----SESSION (same numbers within session) -----")
 with Pyro4.Proxy("PYRONAME:instance.session") as p:
     print(p.msg("hello1"))
     print(p.msg("hello1"))
@@ -20,7 +20,7 @@ with Pyro4.Proxy("PYRONAME:instance.session") as p:
     print(p.msg("hello2"))
     print(p.msg("hello2"))
 
-print("\n-----SINGLE-----")
+print("\n-----SINGLE (same number always) -----")
 with Pyro4.Proxy("PYRONAME:instance.single") as p:
     print(p.msg("hello1"))
     print(p.msg("hello1"))
@@ -30,10 +30,3 @@ with Pyro4.Proxy("PYRONAME:instance.single") as p:
     print(p.msg("hello2"))
     print(p.msg("hello2"))
 
-print("\n-----OLDSTYLE (=single)-----")
-with Pyro4.Proxy("PYRONAME:instance.oldstyle") as p:
-    print(p.msg("hello1"))
-with Pyro4.Proxy("PYRONAME:instance.oldstyle") as p:
-    print(p.msg("hello2"))
-with Pyro4.Proxy("PYRONAME:instance.oldstyle") as p:
-    print(p.msg("hello3"))
