@@ -2,22 +2,17 @@ from __future__ import print_function
 import Pyro4
 
 
-@Pyro4.expose
-class OldStyle(object):
-    def msg(self, message):
-        print("[%s] %s.msg: %s" % (id(self), self.__class__.__name__, message))
-        return id(self)
-
-
-@Pyro4.expose(instance_mode="single")
+@Pyro4.behavior(instance_mode="single")
 class SingleInstance(object):
+    @Pyro4.expose
     def msg(self, message):
         print("[%s] %s.msg: %s" % (id(self), self.__class__.__name__, message))
         return id(self)
 
 
-@Pyro4.expose(instance_mode="session", instance_creator=lambda clazz: clazz.create_instance())
+@Pyro4.behavior(instance_mode="session", instance_creator=lambda clazz: clazz.create_instance())
 class SessionInstance(object):
+    @Pyro4.expose
     def msg(self, message):
         print("[%s] %s.msg: %s" % (id(self), self.__class__.__name__, message))
         return id(self), self.correlation_id
@@ -28,8 +23,9 @@ class SessionInstance(object):
         return obj
 
 
-@Pyro4.expose(instance_mode="percall")
+@Pyro4.behavior(instance_mode="percall")
 class PercallInstance(object):
+    @Pyro4.expose
     def msg(self, message):
         print("[%s] %s.msg: %s" % (id(self), self.__class__.__name__, message))
         return id(self)
@@ -38,7 +34,6 @@ class PercallInstance(object):
 if __name__ == "__main__":
     # please make sure a name server is running somewhere first.
     Pyro4.Daemon.serveSimple({
-        OldStyle: "instance.oldstyle",
         SingleInstance: "instance.single",
         SessionInstance: "instance.session",
         PercallInstance: "instance.percall"
