@@ -1,8 +1,8 @@
 from __future__ import print_function
 import Pyro4
-import person
 
 
+@Pyro4.expose
 class Warehouse(object):
     def __init__(self):
         self.contents = ["chair", "bike", "flashlight", "laptop", "couch"]
@@ -20,12 +20,12 @@ class Warehouse(object):
 
 
 def main():
-    warehouse = Warehouse()
     Pyro4.Daemon.serveSimple(
-            {
-                warehouse: "example.warehouse"
-            },
-            ns = True)
+        {
+            Warehouse: "example.warehouse"
+        },
+        ns=True)
 
-if __name__=="__main__":
+
+if __name__ == "__main__":
     main()
