@@ -132,9 +132,6 @@ def receiveData(sock, size):
         retrydelay = 0.0
         msglen = 0
         chunks = []
-        EMPTY_BYTES = b""
-        if sys.platform == "cli":
-            EMPTY_BYTES = ""
         if Pyro4.config.USE_MSG_WAITALL:
             # waitall is very convenient and if a socket error occurs,
             # we can assume the receive has failed. No need for a loop,
@@ -170,7 +167,7 @@ def receiveData(sock, size):
                         break
                     chunks.append(chunk)
                     msglen += len(chunk)
-                data = EMPTY_BYTES.join(chunks)
+                data = b"".join(chunks)
                 del chunks
                 if len(data) != size:
                     err = ConnectionClosedError("receiving: not enough data")
@@ -314,8 +311,10 @@ def createSocket(bind=None, connect=None, reuseaddr=False, keepalive=True, timeo
                     if sock in sw:
                         break  # yay, writable now, connect() completed
                     elif sock in se:
+                        sock.close()  # close the socket that refused to connect
                         raise socket.error("connect failed")
             else:
+                sock.close()  # close the socket that refused to connect
                 raise
     if keepalive:
         setKeepalive(sock)
@@ -439,6 +438,12 @@ class SocketConnection(object):
     def __del__(self):
         self.close()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.close()
+
     def send(self, data):
         sendData(self.sock, data)
 
@@ -448,11 +453,11 @@ class SocketConnection(object):
     def close(self):
         try:
             self.sock.shutdown(socket.SHUT_RDWR)
-        except (OSError, socket.error):
+        except:
             pass
         try:
             self.sock.close()
-        except AttributeError:
+        except:
             pass
         self.pyroInstances = {}   # force releasing the session instances
 
@@ -472,12 +477,13 @@ def findProbablyUnusedPort(family=socket.AF_INET, socktype=socket.SOCK_STREAM):
     """Returns an unused port that should be suitable for binding (likely, but not guaranteed).
     This code is copied from the stdlib's test.test_support module."""
     tempsock = socket.socket(family, socktype)
-    port = bindOnUnusedPort(tempsock)
-    tempsock.close()
-    del tempsock
-    if sys.platform == "cli":
-        return port + 1  # the actual port is somehow still in use by the socket when using IronPython
-    return port
+    try:
+        port = bindOnUnusedPort(tempsock)
+        if sys.platform == "cli":
+            return port + 1  # the actual port is somehow still in use by the socket when using IronPython
+        return port
+    finally:
+        tempsock.close()
 
 
 def bindOnUnusedPort(sock, host='localhost'):
@@ -504,19 +510,18 @@ def bindOnUnusedPort(sock, host='localhost'):
     return sock.getsockname()[1]
 
 
-"""is select() available?"""
-hasSelect = select and hasattr(select, "select")
-
-"""is poll() available?"""
-hasPoll = select and hasattr(select, "poll")
-
-
-def triggerSocket(sock):
-    """send a small data packet over the socket, to trigger it"""
+def interruptSocket(address):
+    """bit of a hack to trigger a blocking server to get out of the loop, useful at clean shutdowns"""
     try:
-        data = b"!" * 16
-        if sys.platform == "cli":
-            data = "!" * 16
-        sock.sendall(data)
-    except (socket.error, AttributeError):
+        sock = createSocket(connect=address, keepalive=False, timeout=None)
+        try:
+            sock.sendall(b"!" * 16)
+        except (socket.error, AttributeError):
+            pass
+        try:
+            sock.shutdown(socket.SHUT_RDWR)
+        except (OSError, socket.error):
+            pass
+        sock.close()
+    except socket.error:
         pass
