@@ -4,7 +4,6 @@ Tests for the core logic.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
-from __future__ import with_statement
 import copy
 import logging
 import os
@@ -12,6 +11,7 @@ import sys
 import time
 import uuid
 import unittest
+import warnings
 import Pyro4.configuration
 import Pyro4.core
 import Pyro4.errors
@@ -29,9 +29,17 @@ elif sys.version_info >= (3, 4):
 
 
 class CoreTests(unittest.TestCase):
+
+    def setUp(self):
+        warnings.filterwarnings("ignore")
+
+    def tearDown(self):
+        warnings.resetwarnings()
+
     def testProxyNoHmac(self):
         # check that proxy without hmac is possible
-        _ = Pyro4.Proxy("PYRO:object@host:9999")
+        with Pyro4.Proxy("PYRO:object@host:9999") as p:
+            pass
 
     def testDaemonNoHmac(self):
         # check that daemon without hmac is possible
@@ -46,6 +54,16 @@ class CoreTests(unittest.TestCase):
         self.assertIn("COMPRESSION", config)
         self.assertEqual(Pyro4.config.COMPRESSION, config["COMPRESSION"])
 
+    def testConfigDefaults(self):
+        # some security sensitive settings:
+        Pyro4.config.reset(False)   # reset the config to default
+        self.assertTrue(Pyro4.config.REQUIRE_EXPOSE)
+        self.assertEqual("localhost", Pyro4.config.HOST)
+        self.assertEqual("localhost", Pyro4.config.NS_HOST)
+        self.assertFalse(Pyro4.config.FLAME_ENABLED)
+        self.assertEqual("serpent", Pyro4.config.SERIALIZER)
+        self.assertEqual({"json", "serpent", "marshal"}, Pyro4.config.SERIALIZERS_ACCEPTED)
+
     def testConfigValid(self):
         try:
             Pyro4.config.XYZ_FOOBAR = True  # don't want to allow weird config names
@@ -97,7 +115,7 @@ class CoreTests(unittest.TestCase):
         uri = "PYRONAME:some_obj_name@host.com:8888"
         p = Pyro4.core.URI(uri)
         self.assertEqual(uri, str(p))
-        expected = "<Pyro4.core.URI at 0x%x, PYRONAME:some_obj_name@host.com:8888>" % id(p)
+        expected = "<Pyro4.core.URI at 0x%x; PYRONAME:some_obj_name@host.com:8888>" % id(p)
         self.assertEqual(expected, repr(p))
         uri = "PYRO:12345@host.com:9999"
         p = Pyro4.core.URI(uri)
@@ -176,10 +194,32 @@ class CoreTests(unittest.TestCase):
         p = Pyro4.core.URI("pyroname:some_obj_name@host.com:9999")
         self.assertEqual("PYRONAME", p.protocol)
 
+    def testUriParsingPyrometa(self):
+        p = Pyro4.core.URI("PYROMETA:meta")
+        self.assertEqual("PYROMETA", p.protocol)
+        self.assertEqual({"meta"}, p.object)
+        self.assertIsNone(p.host)
+        self.assertIsNone(p.port)
+        p = Pyro4.core.URI("PYROMETA:meta1,meta2,meta2@nameserverhost")
+        self.assertEqual("PYROMETA", p.protocol)
+        self.assertEqual({"meta1", "meta2"}, p.object)
+        self.assertEqual("nameserverhost", p.host)
+        self.assertEqual(Pyro4.config.NS_PORT, p.port)  # PyroMeta uri with host gets a port too if not specified
+        p = Pyro4.core.URI("PYROMETA:meta@nameserverhost:4444")
+        self.assertEqual("PYROMETA", p.protocol)
+        self.assertEqual({"meta"}, p.object)
+        self.assertEqual("nameserverhost", p.host)
+        self.assertEqual(4444, p.port)
+        p = Pyro4.core.URI("PyroMeta:meta1,meta2@host.com:9999")
+        self.assertEqual("PYROMETA", p.protocol)
+        p = Pyro4.core.URI("PyroMeta:meta1,meta2@host.com:9999")
+        self.assertEqual("PYROMETA", p.protocol)
+
     def testInvalidUris(self):
         self.assertRaises(TypeError, Pyro4.core.URI, None)
         self.assertRaises(TypeError, Pyro4.core.URI, 99999)
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "")
+        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, " ")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "a")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYR")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO")
@@ -191,12 +231,18 @@ class CoreTests(unittest.TestCase):
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO:@hostname:portstr")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO:@hostname:7766")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO:objid@hostname:7766:bogus")
+        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO:obj id@hostname:7766")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYROLOC:objname")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYROLOC:objname@host")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYROLOC:objectname@hostname:4444")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRONAME:")
+        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRONAME:obj name@nameserver:bogus")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRONAME:objname@nameserver:bogus")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRONAME:objname@nameserver:7766:bogus")
+        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYROMETA:")
+        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYROMETA:meta@nameserver:bogus")
+        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYROMETA:meta@nameserver:7766:bogus")
+        self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYROMETA:meta1, m2 ,m3@nameserver:7766:bogus")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "FOOBAR:")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "FOOBAR:objid@hostname:7766")
         self.assertRaises(Pyro4.errors.PyroError, Pyro4.core.URI, "PYRO:12345@./u:sockname:9999")
@@ -226,11 +272,11 @@ class CoreTests(unittest.TestCase):
         self.assertEqual("weirdchars" + unichr(0x20AC), pu.object)
         if sys.version_info <= (3, 0):
             self.assertEqual("PYRO:weirdchars?@host?.com:4444", pu.__str__())
-            expected = "<Pyro4.core.URI at 0x%x, PYRO:weirdchars?@host?.com:4444>" % id(pu)
+            expected = "<Pyro4.core.URI at 0x%x; PYRO:weirdchars?@host?.com:4444>" % id(pu)
             self.assertEqual(expected, repr(pu))
         else:
             self.assertEqual("PYRO:weirdchars" + unichr(0x20ac) + "@host" + unichr(0x20ac) + ".com:4444", pu.__str__())
-            expected = ("<Pyro4.core.URI at 0x%x, PYRO:weirdchars" + unichr(0x20ac) + "@host" + unichr(0x20ac) + ".com:4444>") % id(pu)
+            expected = ("<Pyro4.core.URI at 0x%x; PYRO:weirdchars" + unichr(0x20ac) + "@host" + unichr(0x20ac) + ".com:4444>") % id(pu)
             self.assertEqual(expected, repr(pu))
         self.assertEqual("PYRO:weirdchars" + unichr(0x20ac) + "@host" + unichr(0x20ac) + ".com:4444", pu.asString())
         self.assertEqual("PYRO:weirdchars" + unichr(0x20ac) + "@host" + unichr(0x20ac) + ".com:4444", unicode(pu))
@@ -312,6 +358,8 @@ class CoreTests(unittest.TestCase):
         self.assertEqual(p1._pyroTimeout, p2._pyroTimeout)
         self.assertEqual(p1._pyroHmacKey, p2._pyroHmacKey)
         self.assertEqual(p1._pyroHandshake, p2._pyroHandshake)
+        p1._pyroRelease()
+        p2._pyroRelease()
 
     def testProxySubclassCopy(self):
         class ProxySub(Pyro4.core.Proxy):
@@ -319,23 +367,25 @@ class CoreTests(unittest.TestCase):
         p = ProxySub("PYRO:12345@hostname:9999")
         p2 = copy.copy(p)
         self.assertIsInstance(p2, ProxySub)
+        p._pyroRelease()
+        p2._pyroRelease()
 
     def testAsyncProxyAdapterCopy(self):
         try:
             Pyro4.config.METADATA = False
-            proxy = Pyro4.core.Proxy("PYRO:12345@hostname:9999")
-            asyncproxy = proxy._pyroAsync()
-            p2 = copy.copy(asyncproxy)
-            asynccall = p2.foobar()
-            self.assertIsInstance(asynccall, Pyro4.futures.FutureResult)
+            with Pyro4.core.Proxy("PYRO:12345@hostname:9999") as proxy:
+                asyncproxy = proxy._pyroAsync()
+                p2 = copy.copy(asyncproxy)
+                asynccall = p2.foobar()
+                self.assertIsInstance(asynccall, Pyro4.futures.FutureResult)
         finally:
             Pyro4.config.METADATA = True
 
     def testBatchProxyAdapterCopy(self):
-        proxy = Pyro4.core.Proxy("PYRO:12345@hostname:9999")
-        batchproxy = proxy._pyroBatch()
-        p2 = copy.copy(batchproxy)
-        self.assertIsInstance(p2, Pyro4.core._BatchProxyAdapter)
+        with Pyro4.core.Proxy("PYRO:12345@hostname:9999") as proxy:
+            batchproxy = proxy._pyroBatch()
+            p2 = copy.copy(batchproxy)
+            self.assertIsInstance(p2, Pyro4.core._BatchProxyAdapter)
 
     def testProxyOffline(self):
         # only offline stuff here.
@@ -353,13 +403,38 @@ class CoreTests(unittest.TestCase):
         self.assertIsNone(p1._pyroConnection)
         self.assertEqual(p3._pyroUri, p1._pyroUri)
         self.assertIsNot(p3._pyroUri, p1._pyroUri)
+        p3._pyroRelease()
 
     def testProxyRepr(self):
-        p = Pyro4.core.Proxy("PYRO:9999@localhost:15555")
-        address = id(p)
-        expected = "<Pyro4.core.Proxy at 0x%x, not connected, for PYRO:9999@localhost:15555>" % address
-        self.assertEqual(expected, repr(p))
-        self.assertEqual(unicode(expected), unicode(p))
+        with Pyro4.core.Proxy("PYRO:9999@localhost:15555") as p:
+            address = id(p)
+            expected = "<Pyro4.core.Proxy at 0x%x; not connected; for PYRO:9999@localhost:15555>" % address
+            self.assertEqual(expected, repr(p))
+            self.assertEqual(unicode(expected), unicode(p))
+
+    def testProxySerializerOverride(self):
+        serializer = Pyro4.config.SERIALIZER
+        try:
+            Pyro4.config.SERIALIZER = "~invalid~"
+            _ = Pyro4.core.Proxy("PYRO:obj@localhost:5555")
+            self.fail("must raise exception")
+        except Pyro4.errors.SerializeError as x:
+            self.assertIn("~invalid~", str(x))
+            self.assertIn("unknown", str(x))
+        finally:
+            Pyro4.config.SERIALIZER = serializer
+        try:
+            proxy = Pyro4.core.Proxy("PYRO:obj@localhost:5555")
+            proxy._pyroSerializer = "~invalidoverride~"
+            proxy._pyroConnection = "FAKE"
+            proxy.methodcall()
+            self.fail("must raise exception")
+        except Pyro4.errors.SerializeError as x:
+            self.assertIn("~invalidoverride~", str(x))
+            self.assertIn("unknown", str(x))
+        finally:
+            proxy._pyroConnection = None
+            Pyro4.config.SERIALIZER = serializer
 
     def testProxyDir(self):
         # PyPy tries to call deprecated __members__ and __methods__
@@ -382,18 +457,19 @@ class CoreTests(unittest.TestCase):
             self.assertEqual(dir_result, dir(p))
         finally:
             setattr(Pyro4.core.Proxy, '__dir__', old_dir_method)
+        p._pyroRelease()
 
     def testProxyDirMetadata(self):
-        p = Pyro4.core.Proxy("PYRO:9999@localhost:15555")
-        # metadata isn't loaded
-        self.assertIn('__hash__', dir(p))
-        self.assertNotIn('ping', dir(p))
-        # emulate obtaining metadata
-        p._pyroAttrs = set(['prop'])
-        p._pyroMethods = set(['ping'])
-        self.assertIn('__hash__', dir(p))
-        self.assertIn('prop', dir(p))
-        self.assertIn('ping', dir(p))
+        with Pyro4.core.Proxy("PYRO:9999@localhost:15555") as p:
+            # metadata isn't loaded
+            self.assertIn('__hash__', dir(p))
+            self.assertNotIn('ping', dir(p))
+            # emulate obtaining metadata
+            p._pyroAttrs = {"prop"}
+            p._pyroMethods = {"ping"}
+            self.assertIn('__hash__', dir(p))
+            self.assertIn('prop', dir(p))
+            self.assertIn('ping', dir(p))
 
     def testProxySettings(self):
         p1 = Pyro4.core.Proxy("PYRO:9999@localhost:15555")
@@ -410,6 +486,8 @@ class CoreTests(unittest.TestCase):
         self.assertIsNot(p1._pyroOneway, p2._pyroOneway, "p1 and p2 should have different oneway tables")
         self.assertIsNot(p1._pyroAttrs, p2._pyroAttrs, "p1 and p2 should have different attr tables")
         self.assertIsNot(p1._pyroMethods, p2._pyroMethods, "p1 and p2 should have different method tables")
+        p1._pyroRelease()
+        p2._pyroRelease()
 
     def testProxyWithStmt(self):
         class ConnectionMock(object):
@@ -447,6 +525,7 @@ class CoreTests(unittest.TestCase):
             self.assertIsNotNone(p._pyroUri)
         with p:
             self.assertIsNotNone(p._pyroUri)
+        p._pyroRelease()
 
     def testProxyHmac(self):
         class ConnectionMock(object):
@@ -464,10 +543,7 @@ class CoreTests(unittest.TestCase):
         proxy._pyroConnection = conn_mock
         with self.assertRaises(Pyro4.errors.ConnectionClosedError):
             proxy.foo()
-        if sys.platform == 'cli':
-            self.assertIn("HMAC", conn_mock.msgbytes)
-        else:
-            self.assertIn(b"HMAC", conn_mock.msgbytes)
+        self.assertIn(b"HMAC", conn_mock.msgbytes)
 
         conn_mock = ConnectionMock()
         proxy._pyroConnection = conn_mock
@@ -476,6 +552,7 @@ class CoreTests(unittest.TestCase):
             proxy.foo()
         self.assertNotIn(b"HMAC", conn_mock.msgbytes)
         self.assertIsNone(proxy._pyroHmacKey)
+        proxy._pyroRelease()
 
     def testNoConnect(self):
         wrongUri = Pyro4.core.URI("PYRO:foobar@localhost:59999")
@@ -514,6 +591,7 @@ class CoreTests(unittest.TestCase):
         self.assertIsNone(p._pyroTimeout)
         self.assertIsNone(p._pyroConnection.timeout)
         Pyro4.config.COMMTIMEOUT = None
+        p._pyroRelease()
 
     def testCallbackDecorator(self):
         # just test the decorator itself, testing the callback
@@ -542,37 +620,10 @@ class CoreTests(unittest.TestCase):
         self.assertFalse(hash(p1) == hash(p3))
         self.assertFalse(p1 == 42)
         self.assertTrue(p1 != 42)
+        p1._pyroRelease()
+        p2._pyroRelease()
+        p3._pyroRelease()
 
-    def testExposeInstancemodeInvalid(self):
-        with self.assertRaises(ValueError):
-            @Pyro4.core.expose(instance_mode="kaputt")
-            class TestClass:
-                def method(self):
-                    pass
-
-    def testExposeInstancemodeDefault(self):
-        @Pyro4.core.expose()
-        class TestClass:
-            def method(self):
-                pass
-
-    def testExposeInstancecreatorInvalid(self):
-        with self.assertRaises(TypeError):
-            @Pyro4.core.expose(instance_creator=12345)
-            class TestClass:
-                def method(self):
-                    pass
-
-    def testExposeInstancing(self):
-        def creator(clazz):
-            return clazz()
-        @Pyro4.core.expose(instance_mode="percall", instance_creator=creator)
-        class TestClass:
-            def method(self):
-                pass
-        im, ic = TestClass._pyroInstancing
-        self.assertEqual("percall", im)
-        self.assertIs(creator, ic)
 
     def testCallContext(self):
         ctx = Pyro4.core.current_context
@@ -588,6 +639,95 @@ class CoreTests(unittest.TestCase):
         Pyro4.current_context.correlation_id = None
 
 
+class ExposeDecoratorTests(unittest.TestCase):
+    def testExposeInstancemodeDefault(self):
+        @Pyro4.core.expose
+        class TestClassOne:
+            def method(self):
+                pass
+        class TestClassTwo:
+            @Pyro4.core.expose
+            def method(self):
+                pass
+        class TestClassThree:
+            def method(self):
+                pass
+        with Pyro4.core.Daemon() as daemon:
+            daemon.register(TestClassOne)
+            daemon.register(TestClassTwo)
+            daemon.register(TestClassThree)
+            self.assertEqual(("session", None), TestClassOne._pyroInstancing)
+            self.assertEqual(("session", None), TestClassTwo._pyroInstancing)
+            self.assertEqual(("session", None), TestClassThree._pyroInstancing)
+
+
+class BehaviorDecoratorTests(unittest.TestCase):
+    def testBehaviorInstancemodeInvalid(self):
+        with self.assertRaises(ValueError):
+            @Pyro4.core.behavior(instance_mode="kaputt")
+            class TestClass:
+                def method(self):
+                    pass
+
+    def testBehaviorRequiresParams(self):
+        with self.assertRaises(SyntaxError) as x:
+            @Pyro4.core.behavior
+            class TestClass:
+                def method(self):
+                    pass
+        self.assertIn("is missing argument", str(x.exception))
+
+    def testBehaviorInstancecreatorInvalid(self):
+        with self.assertRaises(TypeError):
+            @Pyro4.core.behavior(instance_creator=12345)
+            class TestClass:
+                def method(self):
+                    pass
+
+    def testBehaviorOnMethodInvalid(self):
+        with self.assertRaises(TypeError):
+            class TestClass:
+                @Pyro4.core.behavior(instance_mode="~invalidmode~")
+                def method(self):
+                    pass
+        with self.assertRaises(TypeError):
+            class TestClass:
+                @Pyro4.core.behavior(instance_mode="percall", instance_creator=float)
+                def method(self):
+                    pass
+        with self.assertRaises(TypeError):
+            class TestClass:
+                @Pyro4.core.behavior()
+                def method(self):
+                    pass
+
+    def testBehaviorInstancing(self):
+        @Pyro4.core.behavior(instance_mode="percall", instance_creator=float)
+        class TestClass:
+            def method(self):
+                pass
+        im, ic = TestClass._pyroInstancing
+        self.assertEqual("percall", im)
+        self.assertIs(float, ic)
+
+    def testBehaviorWithExposeKeepsCorrectValues(self):
+        @Pyro4.behavior(instance_mode="percall", instance_creator=float)
+        @Pyro4.expose
+        class TestClass:
+            pass
+        im, ic = TestClass._pyroInstancing
+        self.assertEqual("percall", im)
+        self.assertIs(float, ic)
+
+        @Pyro4.expose
+        @Pyro4.behavior(instance_mode="percall", instance_creator=float)
+        class TestClass2:
+            pass
+        im, ic = TestClass2._pyroInstancing
+        self.assertEqual("percall", im)
+        self.assertIs(float, ic)
+
+
 class RemoteMethodTests(unittest.TestCase):
     class BatchProxyMock(object):
         def __init__(self):
@@ -662,6 +802,7 @@ class RemoteMethodTests(unittest.TestCase):
             self.assertIsInstance(a, Pyro4.core._RemoteMethod, "attribute access should just be a RemoteMethod")
             a2 = a.nestedattribute.nested2
             self.assertIsInstance(a2, Pyro4.core._RemoteMethod, "nested attribute should just be another RemoteMethod")
+            p._pyroRelease()
         finally:
             Pyro4.config.METADATA = True
 
@@ -669,6 +810,7 @@ class RemoteMethodTests(unittest.TestCase):
         p = Pyro4.core.Proxy("PYRO:obj@localhost:59999")
         with self.assertRaises(Pyro4.errors.CommunicationError):
             _ = p.someattribute     # triggers attempt to get metadata
+        p._pyroRelease()
 
     def testBatchMethod(self):
         proxy = self.BatchProxyMock()
@@ -688,6 +830,7 @@ class RemoteMethodTests(unittest.TestCase):
         self.assertRaises(ValueError, next, results)  # the call to error() should generate an exception
         self.assertRaises(StopIteration, next, results)  # and now there should not be any more results
         self.assertEqual(4, len(proxy.result))  # should have done 4 calls, not 5
+        batch._pyroRelease()
 
     def testBatchMethodOneway(self):
         proxy = self.BatchProxyMock()
@@ -750,6 +893,7 @@ class RemoteMethodTests(unittest.TestCase):
         self.assertFalse(result.ready)
         _ = result.value
         self.assertTrue(result.ready)
+        proxy._pyroRelease()
 
     def testAsyncCallbackMethod(self):
         class AsyncFunctionHolder(object):
@@ -769,6 +913,7 @@ class RemoteMethodTests(unittest.TestCase):
         value = result.value
         self.assertEqual(10 // 2 + 2 + 4 + 1, value)
         self.assertEqual(3, holder.asyncFunctionCount)
+        proxy._pyroRelease()
 
     def testCrashingAsyncCallbackMethod(self):
         def normalAsyncFunction(value, x):
@@ -786,6 +931,7 @@ class RemoteMethodTests(unittest.TestCase):
             self.fail("expected exception")
         except ZeroDivisionError:
             pass  # ok
+        proxy._pyroRelease()
 
     def testAsyncMethodTimeout(self):
         proxy = self.AsyncProxyMock()
@@ -796,6 +942,7 @@ class RemoteMethodTests(unittest.TestCase):
         self.assertTrue(result.wait(1))  # will be ready within 1 seconds more
         self.assertTrue(result.ready)
         self.assertEqual(5, result.value)
+        proxy._pyroRelease()
 
 
 class TestSimpleServe(unittest.TestCase):
@@ -806,8 +953,8 @@ class TestSimpleServe(unittest.TestCase):
 
     def testSimpleServe(self):
         with TestSimpleServe.DaemonWrapper() as d:
-            o1 = MyThing(1)
-            o2 = MyThing(2)
+            o1 = MyThingPartlyExposed(1)
+            o2 = MyThingPartlyExposed(2)
             objects = {o1: "test.o1", o2: None}
             Pyro4.core.Daemon.serveSimple(objects, daemon=d, ns=False, verbose=False)
             self.assertEqual(3, len(d.objectsById))
@@ -817,9 +964,9 @@ class TestSimpleServe(unittest.TestCase):
 
     def testSimpleServeSameNames(self):
         with TestSimpleServe.DaemonWrapper() as d:
-            o1 = MyThing(1)
-            o2 = MyThing(2)
-            o3 = MyThing(3)
+            o1 = MyThingPartlyExposed(1)
+            o2 = MyThingPartlyExposed(2)
+            o3 = MyThingPartlyExposed(3)
             objects = {o1: "test.name", o2: "test.name", o3: "test.othername"}
             with self.assertRaises(Pyro4.errors.DaemonError):
                 Pyro4.core.Daemon.serveSimple(objects, daemon=d, ns=False, verbose=False)
@@ -909,6 +1056,54 @@ class TestFutures(unittest.TestCase):
         result = f(4, 5).then(futurestestfunc, 6).then(futurestestfunc, 7).then(futurestestfunc, 8).then(futurestestfunc, 9).then(futurestestfunc, 10)
         value = result.value
         self.assertEqual(4 + 5 + 6 + 7 + 8 + 9 + 10, value)
+        with self.assertRaises(RuntimeError):
+            f(4, 5)   # cannot evaluate the same future more than once
+
+    def testFutureDelay(self):
+        f = Pyro4.Future(futurestestfunc)
+        b = f.delay(0)
+        self.assertTrue(b)
+        begin = time.time()
+        f(4, 5).value
+        duration = time.time() - begin
+        self.assertLess(duration, 0.1)
+        f = Pyro4.Future(futurestestfunc)
+        b = f.delay(1)
+        self.assertTrue(b)
+        begin = time.time()
+        r = f(4, 5)
+        duration = time.time() - begin
+        self.assertLess(duration, 0.1)
+        begin = time.time()
+        r.value
+        duration = time.time() - begin
+        self.assertGreaterEqual(duration, 1)
+        self.assertLess(duration, 1.1)
+        self.assertFalse(f.delay(10))
+
+    def testFutureCancel(self):
+        f = Pyro4.Future(futurestestfunc)
+        f.delay(10)
+        b = f.cancel()
+        self.assertTrue(b)
+        with self.assertRaises(RuntimeError) as x:
+            f(4, 5)
+        self.assertTrue("cancelled" in str(x.exception))
+        f = Pyro4.Future(futurestestfunc)
+        f.delay(10)
+        result = f(4, 5)
+        b = f.cancel()
+        self.assertTrue(b)
+        success = result.wait(3)
+        self.assertTrue(success)
+        with self.assertRaises(RuntimeError) as x:
+            result.value
+        self.assertTrue("cancelled" in str(x.exception))
+        f = Pyro4.Future(futurestestfunc)
+        result = f(4, 5)
+        result.value
+        b = f.cancel()
+        self.assertFalse(b)
 
 
 if __name__ == "__main__":
