@@ -4,13 +4,13 @@ Tests for the daemon.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
-from __future__ import with_statement
 import sys
 import time
 import socket
 import uuid
 import unittest
 import Pyro4.core
+import Pyro4.naming
 import Pyro4.constants
 import Pyro4.socketutil
 import Pyro4.message
@@ -38,19 +38,6 @@ class CustomDaemonInterface(Pyro4.core.DaemonObject):
         return 42
 
 
-class ConnectionMock(object):
-    def __init__(self, initial_msg=None):
-        self.received = initial_msg.to_bytes() if initial_msg else b""
-
-    def send(self, data):
-        self.received += data
-
-    def recv(self, datasize):
-        chunk = self.received[:datasize]
-        self.received = self.received[datasize:]
-        return chunk
-
-
 class DaemonTests(unittest.TestCase):
     # We create a daemon, but notice that we are not actually running the requestloop.
     # 'on-line' tests are all taking place in another test, to keep this one simple.
@@ -87,7 +74,7 @@ class DaemonTests(unittest.TestCase):
                 self.assertIn("serializer that is not accepted", str(x))
                 pass
             import platform
-            if platform.python_implementation() == 'PyPy':
+            if platform.python_implementation() in ('PyPy', 'IronPython'):
                 return
             msg = Pyro4.message.Message(Pyro4.message.MSG_INVOKE, b"", Pyro4.message.SERIALIZER_DILL, 0, 0, hmac_key=d._pyroHmacKey)
             cm = ConnectionMock(msg)
@@ -105,7 +92,7 @@ class DaemonTests(unittest.TestCase):
             self.assertIn(Pyro4.constants.DAEMON_NAME, d.objectsById)
             self.assertEqual("PYRO:" + Pyro4.constants.DAEMON_NAME + "@" + d.locationStr, str(d.uriFor(Pyro4.constants.DAEMON_NAME)))
             # check the string representations
-            expected = "<Pyro4.core.Daemon at 0x%x, %s, 1 objects>" % (id(d), d.locationStr)
+            expected = "<Pyro4.core.Daemon at 0x%x; %s; 1 objects>" % (id(d), d.locationStr)
             self.assertEqual(expected, str(d))
             self.assertEqual(expected, unicode(d))
             self.assertEqual(expected, repr(d))
@@ -133,20 +120,20 @@ class DaemonTests(unittest.TestCase):
             self.assertEqual(locationstr, d.locationStr)
             self.assertEqual("PYRO:" + Pyro4.constants.DAEMON_NAME + "@" + locationstr, str(d.uriFor(Pyro4.constants.DAEMON_NAME)))
             # check the string representations
-            expected = "<Pyro4.core.Daemon at 0x%x, %s, 1 objects>" % (id(d), locationstr)
+            expected = "<Pyro4.core.Daemon at 0x%x; %s; 1 objects>" % (id(d), locationstr)
             self.assertEqual(expected, str(d))
             self.assertEqual(SOCKNAME, d.sock.getsockname())
             self.assertEqual(socket.AF_UNIX, d.sock.family)
 
-    @unittest.skipUnless(hasattr(socket, "AF_UNIX") and sys.platform.startswith("linux"), "linux and unix domain sockets required")
+    @unittest.skipUnless(hasattr(socket, "AF_UNIX") and sys.platform.startswith("linux"), "unix domain sockets required and linux os")
     def testDaemonUnixSocketAbstractNS(self):
-        SOCKNAME = "\0test_unixsocket"  # mind the \0 at the start
+        SOCKNAME = "\0test_unixsocket"  # mind the \0 at the start, for a Linux abstract namespace socket
         with Pyro4.core.Daemon(unixsocket=SOCKNAME) as d:
             locationstr = "./u:" + SOCKNAME
             self.assertEqual(locationstr, d.locationStr)
             self.assertEqual("PYRO:" + Pyro4.constants.DAEMON_NAME + "@" + locationstr, str(d.uriFor(Pyro4.constants.DAEMON_NAME)))
             # check the string representations
-            expected = "<Pyro4.core.Daemon at 0x%x, %s, 1 objects>" % (id(d), locationstr)
+            expected = "<Pyro4.core.Daemon at 0x%x; %s; 1 objects>" % (id(d), locationstr)
             self.assertEqual(expected, str(d))
             sn_bytes = tobytes(SOCKNAME)
             self.assertEqual(sn_bytes, d.sock.getsockname())
@@ -156,8 +143,7 @@ class DaemonTests(unittest.TestCase):
         old_servertype = Pyro4.config.SERVERTYPE
         Pyro4.config.SERVERTYPE = "thread"
         with Pyro4.core.Daemon(port=0) as d:
-            sock = d.sock
-            self.assertIn(sock, d.sockets, "daemon's socketlist should contain the server socket")
+            self.assertIn(d.sock, d.sockets, "daemon's socketlist should contain the server socket")
             self.assertTrue(len(d.sockets) == 1, "daemon without connections should have just 1 socket")
         Pyro4.config.SERVERTYPE = old_servertype
 
@@ -165,8 +151,7 @@ class DaemonTests(unittest.TestCase):
         old_servertype = Pyro4.config.SERVERTYPE
         Pyro4.config.SERVERTYPE = "multiplex"
         with Pyro4.core.Daemon(port=0) as d:
-            sock = d.sock
-            self.assertIn(sock, d.sockets, "daemon's socketlist should contain the server socket")
+            self.assertIn(d.sock, d.sockets, "daemon's socketlist should contain the server socket")
             self.assertTrue(len(d.sockets) == 1, "daemon without connections should have just 1 socket")
         Pyro4.config.SERVERTYPE = old_servertype
 
@@ -182,13 +167,13 @@ class DaemonTests(unittest.TestCase):
             d.register(o1)
             with self.assertRaises(DaemonError) as x:
                 d.register(o1)
-            self.assertEqual("object already has a Pyro id", str(x.exception))
+            self.assertEqual("object or class already has a Pyro id", str(x.exception))
             d.unregister(o1)
             d.register(o1, "samename")
             o2 = MyObj("object2")
             with self.assertRaises(DaemonError) as x:
                 d.register(o2, "samename")
-            self.assertEqual("an object was already registered with that id", str(x.exception))
+            self.assertEqual("an object or class is already registered with that id", str(x.exception))
             self.assertTrue(hasattr(o1, "_pyroId"))
             self.assertTrue(hasattr(o1, "_pyroDaemon"))
             d.unregister(o1)
@@ -197,7 +182,7 @@ class DaemonTests(unittest.TestCase):
             o1._pyroId = "FOOBAR"
             with self.assertRaises(DaemonError) as x:
                 d.register(o1)
-            self.assertEqual("object already has a Pyro id", str(x.exception))
+            self.assertEqual("object or class already has a Pyro id", str(x.exception))
             o1._pyroId = ""
             d.register(o1)  # with empty-string _pyroId register should work
 
@@ -215,8 +200,7 @@ class DaemonTests(unittest.TestCase):
             self.assertIn("newname", d.objectsById)
 
     def testRegisterEtc(self):
-        d = Pyro4.core.Daemon(port=0)
-        try:
+        with Pyro4.core.Daemon(port=0) as d:
             self.assertEqual(1, len(d.objectsById))
             o1 = MyObj("object1")
             o2 = MyObj("object2")
@@ -269,8 +253,17 @@ class DaemonTests(unittest.TestCase):
             uri = d.register(MyObj("xyz"), "test.register")
             self.assertEqual("test.register", uri.object)
 
-        finally:
-            d.close()
+    def testRegisterClass(self):
+        with Pyro4.core.Daemon(port=0) as d:
+            self.assertEqual(1, len(d.objectsById))
+            d.register(MyObj)
+            with self.assertRaises(DaemonError):
+                d.register(MyObj)
+            self.assertEqual(2, len(d.objectsById))
+            d.uriFor(MyObj)
+            # unregister:
+            d.unregister(MyObj)
+            self.assertEqual(1, len(d.objectsById))
 
     def testRegisterUnicode(self):
         with Pyro4.core.Daemon(port=0) as d:
@@ -381,7 +374,7 @@ class DaemonTests(unittest.TestCase):
             start = time.time()
             d.requestLoop(loopCondition=condition)  # this should return almost immediately
             duration = time.time() - start
-            self.assertAlmostEqual(0.0, duration, places=1)
+            self.assertLess(duration, 0.4)
 
     def testSimpleHandshake(self):
         conn = ConnectionMock()
@@ -396,7 +389,7 @@ class DaemonTests(unittest.TestCase):
     def testHandshakeDenied(self):
         class HandshakeFailDaemon(Pyro4.core.Daemon):
             def validateHandshake(self, conn, data):
-                raise ValueError("handshake fail")
+                raise ValueError("handshake fail validation error")
         conn = ConnectionMock()
         with HandshakeFailDaemon(port=0) as d:
             self.sendHandshakeMessage(conn)
@@ -405,7 +398,15 @@ class DaemonTests(unittest.TestCase):
             msg = Pyro4.message.Message.recv(conn, hmac_key=d._pyroHmacKey)
             self.assertEqual(Pyro4.message.MSG_CONNECTFAIL, msg.type)
             self.assertEqual(99, msg.seq)
-            self.assertTrue(b"handshake fail" in msg.data)
+            self.assertTrue(b"handshake fail validation error" in msg.data)
+        with Pyro4.Daemon(port=0) as d:
+            self.sendHandshakeMessage(conn)
+            success = d._handshake(conn, denied_reason="no way, handshake denied")
+            self.assertFalse(success)
+            msg = Pyro4.message.Message.recv(conn, hmac_key=d._pyroHmacKey)
+            self.assertEqual(Pyro4.message.MSG_CONNECTFAIL, msg.type)
+            self.assertEqual(99, msg.seq)
+            self.assertTrue(b"no way, handshake denied" in msg.data)
 
     def testCustomHandshake(self):
         conn = ConnectionMock()
@@ -497,10 +498,19 @@ class DaemonTests(unittest.TestCase):
             Pyro4.config.NATHOST = None
             Pyro4.config.NATPORT = 0
 
+    def testBehaviorDefaults(self):
+        class TestClass:
+            pass
+        with Pyro4.core.Daemon() as d:
+            d.register(TestClass)
+            instance_mode, instance_creator = TestClass._pyroInstancing
+            self.assertEqual("session", instance_mode)
+            self.assertIsNone(instance_creator)
+
     def testInstanceCreationSingle(self):
         def creator(clazz):
             return clazz("testname")
-        @Pyro4.core.expose(instance_mode="single", instance_creator=creator)
+        @Pyro4.core.behavior(instance_mode="single", instance_creator=creator)
         class TestClass:
             def __init__(self, name):
                 self.name = name
@@ -514,55 +524,111 @@ class DaemonTests(unittest.TestCase):
         self.assertIs(instance1, d._pyroInstances[TestClass])
         self.assertFalse(TestClass in conn.pyroInstances)
 
+    def testBehaviorDefaultsIsSession(self):
+        class ClassWithDefaults:
+            def __init__(self):
+                self.name = "yep"
+        conn1 = Pyro4.socketutil.SocketConnection(socket.socket())
+        conn2 = Pyro4.socketutil.SocketConnection(socket.socket())
+        d = Pyro4.core.Daemon()
+        d.register(ClassWithDefaults)
+        instance1a = d._getInstance(ClassWithDefaults, conn1)
+        instance1b = d._getInstance(ClassWithDefaults, conn1)
+        instance2a = d._getInstance(ClassWithDefaults, conn2)
+        instance2b = d._getInstance(ClassWithDefaults, conn2)
+        self.assertIs(instance1a, instance1b)
+        self.assertIs(instance2a, instance2b)
+        self.assertIsNot(instance1a, instance2a)
+        self.assertFalse(ClassWithDefaults in d._pyroInstances)
+        self.assertTrue(ClassWithDefaults in conn1.pyroInstances)
+        self.assertTrue(ClassWithDefaults in conn2.pyroInstances)
+        self.assertIs(instance1a, conn1.pyroInstances[ClassWithDefaults])
+        self.assertIs(instance2a, conn2.pyroInstances[ClassWithDefaults])
+
     def testInstanceCreationSession(self):
         def creator(clazz):
             return clazz("testname")
-        @Pyro4.core.expose(instance_mode="session", instance_creator=creator)
-        class TestClass:
+        @Pyro4.core.behavior(instance_mode="session", instance_creator=creator)
+        class ClassWithDecorator:
             def __init__(self, name):
                 self.name = name
         conn1 = Pyro4.socketutil.SocketConnection(socket.socket())
         conn2 = Pyro4.socketutil.SocketConnection(socket.socket())
         d = Pyro4.core.Daemon()
-        instance1a = d._getInstance(TestClass, conn1)
-        instance1b = d._getInstance(TestClass, conn1)
-        instance2a = d._getInstance(TestClass, conn2)
-        instance2b = d._getInstance(TestClass, conn2)
+        d.register(ClassWithDecorator)
+        # check the class with the decorator first
+        instance1a = d._getInstance(ClassWithDecorator, conn1)
+        instance1b = d._getInstance(ClassWithDecorator, conn1)
+        instance2a = d._getInstance(ClassWithDecorator, conn2)
+        instance2b = d._getInstance(ClassWithDecorator, conn2)
         self.assertIs(instance1a, instance1b)
         self.assertIs(instance2a, instance2b)
         self.assertIsNot(instance1a, instance2a)
-        self.assertFalse(TestClass in d._pyroInstances)
-        self.assertTrue(TestClass in conn1.pyroInstances)
-        self.assertTrue(TestClass in conn2.pyroInstances)
-        self.assertIs(instance1a, conn1.pyroInstances[TestClass])
-        self.assertIs(instance2a, conn2.pyroInstances[TestClass])
+        self.assertFalse(ClassWithDecorator in d._pyroInstances)
+        self.assertTrue(ClassWithDecorator in conn1.pyroInstances)
+        self.assertTrue(ClassWithDecorator in conn2.pyroInstances)
+        self.assertIs(instance1a, conn1.pyroInstances[ClassWithDecorator])
+        self.assertIs(instance2a, conn2.pyroInstances[ClassWithDecorator])
+
 
     def testInstanceCreationPerCall(self):
         def creator(clazz):
             return clazz("testname")
-        @Pyro4.core.expose(instance_mode="percall", instance_creator=creator)
+        @Pyro4.core.behavior(instance_mode="percall", instance_creator=creator)
         class TestClass:
             def __init__(self, name):
                 self.name = name
-        conn = Pyro4.socketutil.SocketConnection(socket.socket())
-        d = Pyro4.core.Daemon()
-        instance1 = d._getInstance(TestClass, conn)
-        instance2 = d._getInstance(TestClass, conn)
-        self.assertIsNot(instance1, instance2)
-        self.assertFalse(TestClass in d._pyroInstances)
-        self.assertFalse(TestClass in conn.pyroInstances)
+        with Pyro4.socketutil.SocketConnection(socket.socket()) as conn:
+            with Pyro4.core.Daemon() as d:
+                instance1 = d._getInstance(TestClass, conn)
+                instance2 = d._getInstance(TestClass, conn)
+                self.assertIsNot(instance1, instance2)
+                self.assertFalse(TestClass in d._pyroInstances)
+                self.assertFalse(TestClass in conn.pyroInstances)
 
     def testInstanceCreationWrongType(self):
         def creator(clazz):
             return Pyro4.core.URI("PYRO:test@localhost:9999")
-        @Pyro4.core.expose(instance_creator=creator)
+        @Pyro4.core.behavior(instance_creator=creator)
         class TestClass:
             def method(self):
                 pass
-        conn = Pyro4.socketutil.SocketConnection(socket.socket())
-        d = Pyro4.core.Daemon()
+        with Pyro4.socketutil.SocketConnection(socket.socket()) as conn:
+            with Pyro4.core.Daemon() as d:
+                with self.assertRaises(TypeError):
+                    d._getInstance(TestClass, conn)
+
+    def testCombine(self):
+        d1 = Pyro4.core.Daemon()
+        d2 = Pyro4.core.Daemon()
         with self.assertRaises(TypeError):
-            d._getInstance(TestClass, conn)
+            d1.combine(d2)
+        d1.close()
+        d2.close()
+        try:
+            Pyro4.config.SERVERTYPE = "multiplex"
+            d1 = Pyro4.core.Daemon()
+            d2 = Pyro4.core.Daemon()
+            nsuri, nsd, bcd = Pyro4.naming.startNS(host="", bchost="")
+            d1_selector = d1.transportServer.selector
+            d1.combine(d2)
+            d1.combine(nsd)
+            d1.combine(bcd)
+            self.assertIs(d1_selector, d1.transportServer.selector)
+            self.assertIs(d1_selector, d2.transportServer.selector)
+            self.assertIs(d1_selector, nsd.transportServer.selector)
+            self.assertIs(d1_selector, bcd.transportServer.selector)
+            self.assertEqual(4, len(d1.sockets))
+            self.assertIn(d1.sock, d1.sockets)
+            self.assertIn(d2.sock, d1.sockets)
+            self.assertIn(nsd.sock, d1.sockets)
+            self.assertIn(bcd, d1.sockets)
+            bcd.close()
+            nsd.close()
+            d2.close()
+            d1.close()
+        finally:
+            Pyro4.config.SERVERTYPE = "thread"
 
 
 class MetaInfoTests(unittest.TestCase):
@@ -571,7 +637,8 @@ class MetaInfoTests(unittest.TestCase):
             daemon_obj = d.objectsById[Pyro4.constants.DAEMON_NAME]
             self.assertTrue(len(daemon_obj.info()) > 10)
             meta = daemon_obj.get_metadata(Pyro4.constants.DAEMON_NAME)
-            self.assertEqual(set(["get_metadata", "info", "ping", "registered"]), meta["methods"])
+            self.assertEqual({"get_metadata", "get_next_stream_item", "close_stream",
+                              "info", "ping", "registered"}, meta["methods"])
 
     def testMetaSerialization(self):
         with Pyro4.core.Daemon() as d:
@@ -579,13 +646,36 @@ class MetaInfoTests(unittest.TestCase):
             meta = daemon_obj.get_metadata(Pyro4.constants.DAEMON_NAME)
             for ser_id in [Pyro4.message.SERIALIZER_JSON, Pyro4.message.SERIALIZER_MARSHAL, Pyro4.message.SERIALIZER_PICKLE, Pyro4.message.SERIALIZER_SERPENT, Pyro4.message.SERIALIZER_DILL]:
                 import platform
-                if platform.python_implementation() == 'PyPy' and \
+                if platform.python_implementation() in ('IronPython', 'PyPy') and \
                    ser_id == Pyro4.message.SERIALIZER_DILL:
                     continue
                 serializer = get_serializer_by_id(ser_id)
                 data = serializer.dumps(meta)
                 _ = serializer.loads(data)
 
+    def testMetaResetCache(self):
+        class Dummy:
+            @Pyro4.expose
+            def method(self):
+                pass
+        with Pyro4.core.Daemon() as d:
+            dummy = Dummy()
+            uri = d.register(dummy)
+            daemon_obj = d.objectsById[Pyro4.constants.DAEMON_NAME]
+            meta = daemon_obj.get_metadata(uri.object)
+            self.assertNotIn("newly_added_method", meta["methods"])
+            self.assertNotIn("newly_added_method_two", meta["methods"])
+            Dummy.newly_added_method = Pyro4.expose(lambda self: None)
+            meta = daemon_obj.get_metadata(uri.object)
+            self.assertNotIn("newly_added_method", meta["methods"])
+            d.resetMetadataCache(uri.object)
+            meta = daemon_obj.get_metadata(uri.object)
+            self.assertIn("newly_added_method", meta["methods"])
+            Dummy.newly_added_method_two = Pyro4.expose(lambda self: None)
+            d.resetMetadataCache(dummy)
+            meta = daemon_obj.get_metadata(uri.object)
+            self.assertIn("newly_added_method_two", meta["methods"])
+
 
 if __name__ == "__main__":
     # import sys;sys.argv = ['', 'Test.testName']
