@@ -36,32 +36,49 @@ class TestEchoserver(unittest.TestCase):
         self.echoserverthread.echodaemon.shutdown()
         time.sleep(0.02)
         self.echoserverthread.join()
+        Pyro4.config.SERVERTYPE = "thread"
 
     def testExposed(self):
         e = Pyro4.test.echoserver.EchoServer()
         self.assertTrue(hasattr(e, "_pyroExposed"))
 
     def testEcho(self):
-        echo = Pyro4.Proxy(self.uri)
-        try:
-            self.assertEqual("hello", echo.echo("hello"))
-            self.assertEqual(None, echo.echo(None))
-            self.assertEqual([1, 2, 3], echo.echo([1, 2, 3]))
-        finally:
-            echo.shutdown()
+        with Pyro4.Proxy(self.uri) as echo:
+            try:
+                self.assertEqual("hello", echo.echo("hello"))
+                self.assertEqual(None, echo.echo(None))
+                self.assertEqual([1, 2, 3], echo.echo([1, 2, 3]))
+            finally:
+                echo.shutdown()
 
     def testError(self):
-        try:
-            echo = Pyro4.Proxy(self.uri)
+        with Pyro4.Proxy(self.uri) as echo:
             try:
                 echo.error()
                 self.fail("expected exception")
-            except:
+            except Exception as x:
                 tb = "".join(Pyro4.util.getPyroTraceback())
                 self.assertIn("Remote traceback", tb)
-                self.assertIn("ZeroDivisionError", tb)
-        finally:
-            echo.shutdown()
+                self.assertIn("ValueError", tb)
+                self.assertEqual("", str(x))
+            try:
+                echo.error_with_text()
+                self.fail("expected exception")
+            except Exception as x:
+                tb = "".join(Pyro4.util.getPyroTraceback())
+                self.assertIn("Remote traceback", tb)
+                self.assertIn("ValueError", tb)
+                self.assertEqual("the message of the error", str(x))
+
+    def testGenerator(self):
+        with Pyro4.Proxy(self.uri) as echo:
+            remotegenerator = echo.generator()
+            self.assertIsInstance(remotegenerator, Pyro4.core._StreamResultIterator)
+            next(remotegenerator)
+            next(remotegenerator)
+            next(remotegenerator)
+            with self.assertRaises(StopIteration):
+                next(remotegenerator)
 
 
 if __name__ == "__main__":
