@@ -12,6 +12,7 @@ from Pyro4.message import Message
 import Pyro4.constants
 import Pyro4.util
 import Pyro4.errors
+from testsupport import ConnectionMock
 
 
 def pyrohmac(data, hmac_key, annotations={}):
@@ -22,19 +23,6 @@ def pyrohmac(data, hmac_key, annotations={}):
     return mac.digest()
 
 
-class ConnectionMock(object):
-    def __init__(self, data=b""):
-        self.received = data
-
-    def send(self, data):
-        self.received += data
-
-    def recv(self, datasize):
-        chunk = self.received[:datasize]
-        self.received = self.received[datasize:]
-        return chunk
-
-
 class MessageTestsHmac(unittest.TestCase):
     def setUp(self):
         self.ser = Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
@@ -167,6 +155,14 @@ class MessageTestsHmac(unittest.TestCase):
         with self.assertRaises(TypeError):
             data.hmac()
 
+    def testSecureCompare(self):
+        self.assertFalse(Pyro4.message.secure_compare("apple", "banana"))
+        self.assertFalse(Pyro4.message.secure_compare(b"apple", b"banana"))
+        self.assertTrue(Pyro4.message.secure_compare("apple", "apple"))
+        self.assertTrue(Pyro4.message.secure_compare(b"apple", b"apple"))
+        with self.assertRaises(TypeError):
+            Pyro4.message.secure_compare(999, "typemismatch")
+
     def testChecksum(self):
         msg = Message(Pyro4.message.MSG_RESULT, b"test", 42, 0, 1, hmac_key=b"secret")
         c = ConnectionMock()
