@@ -4,7 +4,6 @@ Tests for the data serializer.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
-from __future__ import with_statement
 import sys
 import collections
 import copy
@@ -28,6 +27,7 @@ class SerializeTests_pickle(unittest.TestCase):
         self.previous_serializer = Pyro4.config.SERIALIZER
         Pyro4.config.SERIALIZER = self.SERIALIZER
         self.ser = Pyro4.util.get_serializer(Pyro4.config.SERIALIZER)
+        Pyro4.config.REQUIRE_EXPOSE = True
 
     def tearDown(self):
         Pyro4.config.SERIALIZER = self.previous_serializer
@@ -183,6 +183,7 @@ class SerializeTests_pickle(unittest.TestCase):
         proxy._pyroHmacKey = b"secret"
         proxy._pyroHandshake = "apples"
         proxy._pyroMaxRetries = 78
+        proxy._pyroSerializer = "serializer"
         s, c = self.ser.serializeData(proxy)
         x = self.ser.deserializeData(s, c)
         self.assertIsInstance(x, Pyro4.core.Proxy)
@@ -192,6 +193,7 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual(set("ghi"), x._pyroOneway)
         self.assertEqual(b"secret", x._pyroHmacKey)
         self.assertEqual("apples", x._pyroHandshake)
+        self.assertEqual("serializer", x._pyroSerializer)
         self.assertEqual(0, x._pyroTimeout, "must be reset to defaults")
         self.assertEqual(0, x._pyroMaxRetries, "must be reset to defaults")
         daemon = Pyro4.core.Daemon()
@@ -219,12 +221,10 @@ class SerializeTests_pickle(unittest.TestCase):
         proxy._pyroHmacKey = b"secret"
         proxy._pyroHandshake = "apples"
         proxy._pyroMaxRetries = 78
+        proxy._pyroSerializer = "serializer"
         state = proxy.__getstate_for_dict__()
-        if sys.platform == 'cli':
-            b64_secret = "b64:"+base64.b64encode("secret").decode("utf-8")
-        else:
-            b64_secret = "b64:"+base64.b64encode(b"secret").decode("utf-8")
-        self.assertEqual(('PYRO:object@host:4444', tuple(set("ghi")), tuple(set("def")), tuple(set("abc")), 42, b64_secret, "apples", 78), state)
+        b64_secret = "b64:"+base64.b64encode(b"secret").decode("utf-8")
+        self.assertEqual(('PYRO:object@host:4444', tuple(set("ghi")), tuple(set("def")), tuple(set("abc")), 42, b64_secret, "apples", 78, "serializer"), state)
         proxy2 = Pyro4.core.Proxy("PYRONAME:xxx")
         proxy2.__setstate_from_dict__(state)
         self.assertEqual(proxy, proxy2)
@@ -234,6 +234,7 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual(proxy._pyroOneway, proxy2._pyroOneway)
         self.assertEqual(proxy._pyroHmacKey, proxy2._pyroHmacKey)
         self.assertEqual(proxy._pyroHandshake, proxy2._pyroHandshake)
+        self.assertEqual(proxy._pyroSerializer, proxy2._pyroSerializer)
         self.assertEqual(0, proxy2._pyroTimeout, "must be reset to defaults")
         self.assertEqual(0, proxy2._pyroMaxRetries, "must be reset to defaults")
         daemon = Pyro4.core.Daemon()
@@ -242,12 +243,27 @@ class SerializeTests_pickle(unittest.TestCase):
         daemon2 = Pyro4.core.Daemon()
         daemon2.__setstate_from_dict__(state)
 
-    def testAutoProxy(self):
+    def testProxySerializationCompat(self):
+        proxy = Pyro4.core.Proxy("PYRO:object@host:4444")
+        proxy._pyroSerializer = "serializer"
+        pickle_state = proxy.__getstate__()
+        self.assertEqual(9, len(pickle_state))
+        pickle_state = pickle_state[:8]
+        proxy.__setstate__(pickle_state)
+        self.assertIsNone(proxy._pyroSerializer)
+        proxy._pyroSerializer = "serializer"
+        serpent_state = proxy.__getstate_for_dict__()
+        self.assertEqual(9, len(serpent_state))
+        serpent_state = serpent_state[:8]
+        proxy.__setstate_from_dict__(serpent_state)
+        self.assertIsNone(proxy._pyroSerializer)
+
+    def testAutoProxyPartlyExposed(self):
         if self.SERIALIZER == "marshal":
             self.skipTest("marshal can't serialize custom objects")
-        self.ser.register_type_replacement(MyThing, Pyro4.core.pyroObjectToAutoProxy)
-        t1 = MyThing("1")
-        t2 = MyThing("2")
+        self.ser.register_type_replacement(MyThingPartlyExposed, Pyro4.core.pyroObjectToAutoProxy)
+        t1 = MyThingPartlyExposed("1")
+        t2 = MyThingPartlyExposed("2")
         with Pyro4.core.Daemon() as d:
             d.register(t1, "thingy1")
             d.register(t2, "thingy2")
@@ -261,9 +277,32 @@ class SerializeTests_pickle(unittest.TestCase):
             self.assertIsInstance(p2, Pyro4.core.Proxy)
             self.assertEqual("thingy1", p1._pyroUri.object)
             self.assertEqual("thingy2", p2._pyroUri.object)
-            self.assertEqual(set(["prop1", "prop2", "readonly_prop1"]), p1._pyroAttrs)
-            self.assertEqual(set(['classmethod', 'method', 'oneway', 'staticmethod', 'exposed', "__dunder__"]), p1._pyroMethods)
-            self.assertEqual(set(['oneway']), p1._pyroOneway)
+            self.assertEqual({"prop1", "readonly_prop1"}, p1._pyroAttrs)
+            self.assertEqual({"exposed", "oneway"}, p1._pyroMethods)
+            self.assertEqual({'oneway'}, p1._pyroOneway)
+
+    def testAutoProxyFullExposed(self):
+        if self.SERIALIZER == "marshal":
+            self.skipTest("marshal can't serialize custom objects")
+        self.ser.register_type_replacement(MyThingPartlyExposed, Pyro4.core.pyroObjectToAutoProxy)
+        t1 = MyThingFullExposed("1")
+        t2 = MyThingFullExposed("2")
+        with Pyro4.core.Daemon() as d:
+            d.register(t1, "thingy1")
+            d.register(t2, "thingy2")
+            data = [t1, ["apple", t2]]
+            s, c = self.ser.serializeData(data)
+            data = self.ser.deserializeData(s, c)
+            self.assertEqual("apple", data[1][0])
+            p1 = data[0]
+            p2 = data[1][1]
+            self.assertIsInstance(p1, Pyro4.core.Proxy)
+            self.assertIsInstance(p2, Pyro4.core.Proxy)
+            self.assertEqual("thingy1", p1._pyroUri.object)
+            self.assertEqual("thingy2", p2._pyroUri.object)
+            self.assertEqual({"prop1", "prop2", "readonly_prop1"}, p1._pyroAttrs)
+            self.assertEqual({'classmethod', 'method', 'oneway', 'staticmethod', 'exposed', "__dunder__"}, p1._pyroMethods)
+            self.assertEqual({'oneway'}, p1._pyroOneway)
 
     def testCustomClassFail(self):
         if self.SERIALIZER in ("pickle", "dill"):
@@ -279,12 +318,12 @@ class SerializeTests_pickle(unittest.TestCase):
     def testCustomClassOk(self):
         if self.SERIALIZER in ("pickle", "dill"):
             self.skipTest("pickle and dill simply serialize custom classes just fine")
-        o = MyThing("test")
-        Pyro4.util.SerializerBase.register_class_to_dict(MyThing, mything_dict)
+        o = MyThingPartlyExposed("test")
+        Pyro4.util.SerializerBase.register_class_to_dict(MyThingPartlyExposed, mything_dict)
         Pyro4.util.SerializerBase.register_dict_to_class("CUSTOM-Mythingymabob", mything_creator)
         s, c = self.ser.serializeData(o)
         o2 = self.ser.deserializeData(s, c)
-        self.assertIsInstance(o2, MyThing)
+        self.assertIsInstance(o2, MyThingPartlyExposed)
         self.assertEqual("test", o2.name)
         # unregister the deserializer
         Pyro4.util.SerializerBase.unregister_dict_to_class("CUSTOM-Mythingymabob")
@@ -294,15 +333,15 @@ class SerializeTests_pickle(unittest.TestCase):
         except Pyro4.errors.ProtocolError:
             pass  # ok
         # unregister the serializer
-        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThing)
+        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThingPartlyExposed)
         s, c = self.ser.serializeData(o)
         try:
             self.ser.deserializeData(s, c)
             self.fail("must fail")
         except Pyro4.errors.SerializeError as x:
             msg = str(x)
-            self.assertIn(msg, ["unsupported serialized class: testsupport.MyThing",
-                                "unsupported serialized class: PyroTests.testsupport.MyThing"])
+            self.assertIn(msg, ["unsupported serialized class: testsupport.MyThingPartlyExposed",
+                                "unsupported serialized class: PyroTests.testsupport.MyThingPartlyExposed"])
 
     def testData(self):
         data = [42, "hello"]
@@ -314,7 +353,7 @@ class SerializeTests_pickle(unittest.TestCase):
         self.assertEqual(data, data2)
 
     def testSet(self):
-        data = set([111, 222, 333])
+        data = {111, 222, 333}
         ser, compressed = self.ser.serializeData(data)
         data2 = self.ser.deserializeData(ser, compressed=compressed)
         self.assertEqual(data, data2)
@@ -431,10 +470,19 @@ class SerializeTests_pickle(unittest.TestCase):
         finally:
             Pyro4.config.PICKLE_PROTOCOL_VERSION = orig_protocol
 
+    def testFloatPrecision(self):
+        f1 = 1482514078.54635912345
+        f2 = 9876543212345.12345678987654321
+        f3 = 11223344.556677889988776655e33
+        floats = [f1, f2, f3]
+        d, compr = self.ser.serializeData(floats)
+        v = self.ser.deserializeData(d, compr)
+        self.assertEqual(floats, v, "float precision must not be compromised in any serializer")
+
 
 import platform
-@unittest.skipIf(platform.python_implementation() == 'PyPy',
-                 'PyPy not currently supported with dill')
+@unittest.skipIf(platform.python_implementation() in ('PyPy', 'IronPython'),
+                 'PyPy and IronPython not currently supported with dill')
 class SerializeTests_dill(SerializeTests_pickle):
     SERIALIZER = "dill"
 
@@ -473,7 +521,7 @@ class SerializeTests_serpent(SerializeTests_pickle):
 
     def testSet(self):
         # serpent serializes a set into a tuple on older python versions, so we override this
-        data = set([111, 222, 333])
+        data = {111, 222, 333}
         ser, compressed = self.ser.serializeData(data)
         data2 = self.ser.deserializeData(ser, compressed=compressed)
         if serpent.can_use_set_literals:
@@ -528,7 +576,7 @@ class SerializeTests_json(SerializeTests_pickle):
 
     def testSet(self):
         # json serializes a set into a list, so we override this
-        data = set([111, 222, 333])
+        data = {111, 222, 333}
         ser, compressed = self.ser.serializeData(data)
         data2 = self.ser.deserializeData(ser, compressed=compressed)
         self.assertEqual(list(data), data2)
@@ -570,8 +618,8 @@ class GenericTests(unittest.TestCase):
             pass
         try:
             import platform
-            if platform.python_implementation() == 'PyPy':
-                raise ImportError('Currently dill is not supported with PyPy')
+            if platform.python_implementation() in ('PyPy', 'IronPython'):
+                raise ImportError('Currently dill is not supported with IronPython and PyPy')
             import dill
             Pyro4.util.get_serializer("dill")
         except ImportError:
@@ -580,7 +628,7 @@ class GenericTests(unittest.TestCase):
     def testSerializersAvailableById(self):
         Pyro4.util.get_serializer_by_id(Pyro4.message.SERIALIZER_PICKLE)
         import platform
-        if platform.python_implementation() != 'PyPy':
+        if platform.python_implementation() not in ('PyPy', 'IronPython'):
             Pyro4.util.get_serializer_by_id(Pyro4.message.SERIALIZER_DILL)
         Pyro4.util.get_serializer_by_id(Pyro4.message.SERIALIZER_MARSHAL)
         self.assertRaises(Pyro4.errors.ProtocolError, lambda: Pyro4.util.get_serializer_by_id(9999999))
@@ -634,21 +682,21 @@ class GenericTests(unittest.TestCase):
         self.assertEqual("/tmp/socketname", x.sockname)
 
     def testCustomDictClass(self):
-        o = MyThing("test")
-        Pyro4.util.SerializerBase.register_class_to_dict(MyThing, mything_dict)
+        o = MyThingPartlyExposed("test")
+        Pyro4.util.SerializerBase.register_class_to_dict(MyThingPartlyExposed, mything_dict)
         Pyro4.util.SerializerBase.register_dict_to_class("CUSTOM-Mythingymabob", mything_creator)
         d = Pyro4.util.SerializerBase.class_to_dict(o)
         self.assertEqual("CUSTOM-Mythingymabob", d["__class__"])
         self.assertEqual("test", d["name"])
         x = Pyro4.util.SerializerBase.dict_to_class(d)
-        self.assertIsInstance(x, MyThing)
+        self.assertIsInstance(x, MyThingPartlyExposed)
         self.assertEqual("test", x.name)
         # unregister the conversion functions and try again
-        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThing)
+        Pyro4.util.SerializerBase.unregister_class_to_dict(MyThingPartlyExposed)
         Pyro4.util.SerializerBase.unregister_dict_to_class("CUSTOM-Mythingymabob")
         d_orig = Pyro4.util.SerializerBase.class_to_dict(o)
         clsname = d_orig["__class__"]
-        self.assertTrue(clsname.endswith("testsupport.MyThing"))
+        self.assertTrue(clsname.endswith("testsupport.MyThingPartlyExposed"))
         try:
             _ = Pyro4.util.SerializerBase.dict_to_class(d)
             self.fail("should crash")
@@ -694,7 +742,7 @@ def mything_dict(obj):
 def mything_creator(classname, d):
     assert classname == "CUSTOM-Mythingymabob"
     assert d["__class__"] == "CUSTOM-Mythingymabob"
-    return MyThing(d["name"])
+    return MyThingPartlyExposed(d["name"])
 
 
 if __name__ == "__main__":
