@@ -4,7 +4,6 @@ Tests for a running Pyro server, without timeouts.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
-from __future__ import with_statement
 import time
 import sys
 import uuid
@@ -79,6 +78,14 @@ class ServerTestObject(object):
     def dictionary(self):
         return self._dictionary
 
+    def iterator(self):
+        return iter(["one", "two", "three"])
+
+    def generator(self):
+        yield "one"
+        yield "two"
+        yield "three"
+
 
 class NotEverythingExposedClass(object):
     def __init__(self, name):
@@ -109,10 +116,10 @@ class DaemonLoopThread(threadutil.Thread):
 
 
 class DaemonWithSabotagedHandshake(Pyro4.core.Daemon):
-    def _handshake(self, conn):
+    def _handshake(self, conn, denied_reason=None):
         # receive the client's handshake data
         msg = Pyro4.message.Message.recv(conn, [Pyro4.message.MSG_CONNECT], self._pyroHmacKey)
-        # return a CONNECTFAIL
+        # return a CONNECTFAIL always
         serializer = Pyro4.util.get_serializer_by_id(msg.serializer_id)
         data, _ = serializer.serializeData("rigged connection failure", compress=False)
         msg = Pyro4.message.Message(Pyro4.message.MSG_CONNECTFAIL, data, serializer.serializer_id, 0, 1, hmac_key=self._pyroHmacKey)
@@ -276,18 +283,18 @@ class ServerTestsOnce(unittest.TestCase):
             self.assertEqual(set(), p._pyroOneway)
             # connecting it should obtain metadata (as long as METADATA is true)
             p._pyroBind()
-            self.assertEqual(set(['value', 'dictionary']), p._pyroAttrs)
-            self.assertEqual(set(['echo', 'getDict', 'divide', 'nonserializableException', 'ping', 'oneway_delay', 'delayAndId', 'delay', 'testargs',
-                                  'multiply', 'oneway_multiply', 'getDictAttr']), p._pyroMethods)
-            self.assertEqual(set(['oneway_multiply', 'oneway_delay']), p._pyroOneway)
+            self.assertEqual({'value', 'dictionary'}, p._pyroAttrs)
+            self.assertEqual({'echo', 'getDict', 'divide', 'nonserializableException', 'ping', 'oneway_delay', 'delayAndId', 'delay', 'testargs',
+                                  'multiply', 'oneway_multiply', 'getDictAttr', 'iterator', 'generator'}, p._pyroMethods)
+            self.assertEqual({'oneway_multiply', 'oneway_delay'}, p._pyroOneway)
             p._pyroAttrs = None
             p._pyroGetMetadata()
-            self.assertEqual(set(['value', 'dictionary']), p._pyroAttrs)
+            self.assertEqual({'value', 'dictionary'}, p._pyroAttrs)
             p._pyroAttrs = None
             p._pyroGetMetadata(self.objectUri.object)
-            self.assertEqual(set(['value', 'dictionary']), p._pyroAttrs)
+            self.assertEqual({'value', 'dictionary'}, p._pyroAttrs)
             p._pyroAttrs = None
-            p._pyroGetMetadata(known_metadata={"attrs": set(), "oneway": set(), "methods": set(["ping"])})
+            p._pyroGetMetadata(known_metadata={"attrs": set(), "oneway": set(), "methods": {"ping"}})
             self.assertEqual(set(), p._pyroAttrs)
 
     def testProxyAttrsMetadataOff(self):
@@ -355,9 +362,9 @@ class ServerTestsOnce(unittest.TestCase):
             old_require = Pyro4.config.REQUIRE_EXPOSE
             Pyro4.config.REQUIRE_EXPOSE = False
             with self.daemon.proxyFor("unexposed") as p:
-                self.assertEqual(set(["unexposed", "getName"]), p._pyroMethods)
+                self.assertEqual({"unexposed", "getName"}, p._pyroMethods)
                 self.assertEqual("hello", p.getName())
-                self.assertEqual("you should not see this", p.unexposed())   # well, you should see it when REQUIRE_EXPOSE is False :)
+                self.assertEqual("you should not see this", p.unexposed())   # you *should* see it when REQUIRE_EXPOSE is False :)
         finally:
             Pyro4.config.REQUIRE_EXPOSE = old_require
 
@@ -366,7 +373,7 @@ class ServerTestsOnce(unittest.TestCase):
             old_require = Pyro4.config.REQUIRE_EXPOSE
             Pyro4.config.REQUIRE_EXPOSE = True
             with self.daemon.proxyFor("unexposed") as p:
-                self.assertEqual(set(["getName"]), p._pyroMethods)
+                self.assertEqual({"getName"}, p._pyroMethods)
                 self.assertEqual("hello", p.getName())
                 with self.assertRaises(AttributeError) as e:
                     p.unexposed()
@@ -383,7 +390,7 @@ class ServerTestsOnce(unittest.TestCase):
         with Pyro4.core.Proxy(self.objectUri) as p:
             _ = p.value
             # metadata should be loaded now
-            self.assertEqual(set(["value", "dictionary"]), p._pyroAttrs)
+            self.assertEqual({"value", "dictionary"}, p._pyroAttrs)
             with self.assertRaises(AttributeError):
                 _ = p.something
             with self.assertRaises(AttributeError):
@@ -490,7 +497,7 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testAsyncProxyCallchain(self):
         class FuncHolder(object):
-            count = AtomicCounter()
+            count = threadutil.AtomicCounter()
 
             def function(self, value, increase=1):
                 self.count.incr()
@@ -510,7 +517,7 @@ class ServerTestsOnce(unittest.TestCase):
             value = result.value
             self.assertTrue(result.ready)
             self.assertEqual(22, value)
-            self.assertEqual(3, holder.count.value())
+            self.assertEqual(3, holder.count.value)
 
     def testBatchOneway(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
@@ -542,7 +549,7 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testBatchAsyncCallchain(self):
         class FuncHolder(object):
-            count = AtomicCounter()
+            count = threadutil.AtomicCounter()
 
             def function(self, values):
                 result = [value + 1 for value in values]
@@ -559,7 +566,7 @@ class ServerTestsOnce(unittest.TestCase):
             value = result.value
             self.assertTrue(result.ready)
             self.assertEqual([44, 14], value)
-            self.assertEqual(2, holder.count.value())
+            self.assertEqual(2, holder.count.value)
 
     def testPyroTracebackNormal(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
@@ -625,7 +632,7 @@ class ServerTestsOnce(unittest.TestCase):
 
     def testConnectingThreads(self):
         class ConnectingThread(threadutil.Thread):
-            new_connections = AtomicCounter()
+            new_connections = threadutil.AtomicCounter()
 
             def __init__(self, proxy, event):
                 threadutil.Thread.__init__(self)
@@ -647,7 +654,7 @@ class ServerTestsOnce(unittest.TestCase):
             event.set()
             for t in threads:
                 t.join()
-            self.assertEqual(1, ConnectingThread.new_connections.value())  # proxy shared among threads must still have only 1 connect done
+            self.assertEqual(1, ConnectingThread.new_connections.value)  # proxy shared among threads must still have only 1 connect done
 
     def testMaxMsgSize(self):
         with Pyro4.core.Proxy(self.objectUri) as p:
@@ -662,6 +669,28 @@ class ServerTestsOnce(unittest.TestCase):
                 pass
             Pyro4.config.MAX_MESSAGE_SIZE = 0
 
+    def testIterator(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            iterator = p.iterator()
+            self.assertIsInstance(iterator, Pyro4.core._StreamResultIterator)
+            self.assertEqual("one", next(iterator))
+            self.assertEqual("two", next(iterator))
+            self.assertEqual("three", next(iterator))
+            with self.assertRaises(StopIteration):
+                next(iterator)
+            iterator.close()
+
+    def testGenerator(self):
+        with Pyro4.core.Proxy(self.objectUri) as p:
+            generator = p.generator()
+            self.assertIsInstance(generator, Pyro4.core._StreamResultIterator)
+            self.assertEqual("one", next(generator))
+            self.assertEqual("two", next(generator))
+            self.assertEqual("three", next(generator))
+            with self.assertRaises(StopIteration):
+                next(generator)
+            generator.close()
+
     def testCleanup(self):
         p1 = Pyro4.core.Proxy(self.objectUri)
         p2 = Pyro4.core.Proxy(self.objectUri)
@@ -790,7 +819,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
         class ProxyWithOneway(Pyro4.core.Proxy):
             def __init__(self, arg):
                 super(ProxyWithOneway, self).__init__(arg)
-                self._pyroOneway = set(["oneway_multiply", "multiply"])
+                self._pyroOneway = {"oneway_multiply", "multiply"}
 
         with ProxyWithOneway(self.objectUri) as p:
             self.assertIsNone(p.oneway_multiply(5, 11))
@@ -807,6 +836,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 Pyro4.config.ONEWAY_THREADED = True  # the default
                 now = time.time()
                 p.oneway_delay(1)  # oneway so we should continue right away
+                time.sleep(0.01)
                 self.assertTrue(time.time() - now < 0.2, "delay should be running as oneway")
                 now = time.time()
                 self.assertEqual(55, p.multiply(5, 11), "expected a normal result from a non-oneway call")
@@ -816,6 +846,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
                 Pyro4.config.ONEWAY_THREADED = False
                 now = time.time()
                 p.oneway_delay(1)  # oneway so we should continue right away
+                time.sleep(0.01)
                 self.assertTrue(time.time() - now < 0.2, "delay should be running as oneway")
                 now = time.time()
                 self.assertEqual(55, p.multiply(5, 11), "expected a normal result from a non-oneway call")
@@ -879,7 +910,7 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             self.assertRaises(Pyro4.errors.TimeoutError, p.delay, 1)
             duration = time.time() - start
             if sys.platform != "cli":
-                self.assertAlmostEqual(0.1, duration, places=1)
+                self.assertLess(duration, 0.3)
             else:
                 # ironpython's time is weird
                 self.assertTrue(0.0 < duration < 0.7)
@@ -991,6 +1022,57 @@ class ServerTestsThreadNoTimeout(unittest.TestCase):
             # so 6 threads taking 0.5 seconds =~ 0.5 seconds passed
             self.assertTrue(0.4 < duration < 0.9)
 
+    def testGeneratorProxyClose(self):
+        p = Pyro4.core.Proxy(self.objectUri)
+        generator = p.generator()
+        p._pyroRelease()
+        with self.assertRaises(Pyro4.errors.ConnectionClosedError):
+            next(generator)
+
+    def testGeneratorLinger(self):
+        orig_linger = Pyro4.config.ITER_STREAM_LINGER
+        orig_commt = Pyro4.config.COMMTIMEOUT
+        orig_pollt = Pyro4.config.POLLTIMEOUT
+        try:
+            Pyro4.config.ITER_STREAM_LINGER = 0.5
+            Pyro4.config.COMMTIMEOUT = 0.2
+            Pyro4.config.POLLTIMEOUT = 0.2
+            p = Pyro4.core.Proxy(self.objectUri)
+            generator = p.generator()
+            self.assertEqual("one", next(generator))
+            p._pyroRelease()
+            with self.assertRaises(Pyro4.errors.ConnectionClosedError):
+                next(generator)
+            p._pyroReconnect()
+            self.assertEqual("two", next(generator), "generator should resume after reconnect")
+            # check that after the linger time passes, the generator *is* gone
+            p._pyroRelease()
+            time.sleep(2)
+            p._pyroReconnect()
+            with self.assertRaises(Pyro4.errors.PyroError):  # should not be resumable anymore
+                next(generator)
+        finally:
+            Pyro4.config.ITER_STREAM_LINGER = orig_linger
+            Pyro4.config.COMMTIMEOUT = orig_commt
+            Pyro4.config.POLLTIMEOUT = orig_pollt
+
+    def testGeneratorNoLinger(self):
+        orig_linger = Pyro4.config.ITER_STREAM_LINGER
+        try:
+            p = Pyro4.core.Proxy(self.objectUri)
+            Pyro4.config.ITER_STREAM_LINGER = 0  # disable linger
+            generator = p.generator()
+            self.assertEqual("one", next(generator))
+            p._pyroRelease()
+            with self.assertRaises(Pyro4.errors.ConnectionClosedError):
+                next(generator)
+            p._pyroReconnect()
+            with self.assertRaises(Pyro4.errors.PyroError):  # should not be resumable after reconnect
+                next(generator)
+            generator.close()
+        finally:
+            Pyro4.config.ITER_STREAM_LINGER = orig_linger
+
 
 class ServerTestsMultiplexNoTimeout(ServerTestsThreadNoTimeout):
     SERVERTYPE = "multiplex"
